from lib_common import UserService, URLFetch, TastyPieCollection, TastyPieContainer
import urllib, json, numpy, twitter, unicodedata
from time import mktime, strptime, strftime, time, gmtime
from cosine_test import process
from lib_keyword import KeywordExtraction
import nsga

import unicodedata

class Tweet(TastyPieContainer):
	def getLifetime(self):
		if 'tweet_lifetime' in self.getattr('extended'):
			return self.getattr('extended')['tweet_lifetime']
		else:	
			try:
				self.getattr('extended')['tweet_lifetime'] = (int(time()) - int(mktime(strptime(self.created_at, '%a %b %d %H:%M:%S +0000 %Y'))))
				return self.getattr('extended')['tweet_lifetime']
			except:
				return False
				
	def getAuthorAge(self):
		if 'author_account_age' in self.getattr('extended'):
			return self.getattr('extended')['author_account_age']
		else:
			try:		
				self.getattr('extended')['author_account_age'] = ((((int(time()) - int(mktime(strptime(self.getattr('user')['created_at'], '%a %b %d %H:%M:%S +0000 %Y'))))/60)/60)/24)
				return self.getattr('extended')['author_account_age']			
			except:
				return False
				
	def getKeywords(self):
		if 'tweet_keywords' in self.getattr('extended'):
			return self.getattr('extended')['tweet_keywords']
		else:	
			try:
				k = KeywordExtraction()
				extracted = (k.extractLiteral(self.text))
				self.getattr('extended')['tweet_keywords'] = [kw.keyword.lower() for kw in extracted.data.itervalues()]
				return self.getattr('extended')['tweet_keywords']
			except:
				return False
				
	def getPrecision(self, setA, setB):	#move to lib common?
		try:
			setA = set(setA)
			setB = set(setB)
			return len(setB.intersection(setA))/float(len(setA))
		except:
			return False
			
	def getEstablishedScore(self):
		if 'author_establishment_score' in self.getattr('extended'):
			return self.getattr('extended')['author_establishment_score']	
		else:
			try:
				self.getattr('extended')['author_establishment_score']	= self.getAuthorAge()/float(self.getattr('user')['friends_count'])
				return self.getattr('extended')['author_establishment_score']	
			except ZeroDivisionError:
				self.getattr('extended')['author_establishment_score']	= 0
				return self.getattr('extended')['author_establishment_score']	
			except:
				
				return False
				
	def getPostFrequency(self):
		if 'author_post_frequency' in self.getattr('extended'):
			return self.getattr('extended')['author_post_frequency']		
		else:
			try:
				self.getattr('extended')['author_post_frequency'] = round(self.getattr('user')['statuses_count']/float(self.getAuthorAge()),3)
				return self.getattr('extended')['author_post_frequency']
			except ZeroDivisionError:
				self.getattr('extended')['author_post_frequency'] = 0
				return self.getattr('extended')['author_post_frequency']
			except:
				return False
	
	def getReachScore(self):
		if 'author_reach_score' in self.getattr('extended'):
			return self.getattr('extended')['author_reach_score']	
		else:
			try:
				self.getattr('extended')['author_reach_score'] = round(self.getattr('user')['followers_count']/float(self.getPostFrequency()),3)
				return self.getattr('extended')['author_reach_score']
			except ZeroDivisionError:
				self.getattr('extended')['author_reach_score'] = 0
				return self.getattr('extended')['author_reach_score']
			except:
				return False
				
	def getCosineSimilarity(self,againstList=None):
		if 'tweet_cosine_similarity_to_neighbours' in self.getattr('extended'):
			return self.getattr('extended')['tweet_cosine_similarity_to_neighbours']
		else:
			if not isinstance(againstList,list) or len(againstList) == 0:
				return False
			try:
				#print againstList[0]
				cosineAverages = [process(againstText.lower(),self.getattr('text').lower()) if (self.getattr('text') != againstText) else False for againstText in againstList]
				self.getattr('extended')['tweet_cosine_similarity_to_neighbours'] = round(numpy.average(cosineAverages),4)
				#print self.cosine_similarity_to_neighbours
				
				return self.getattr('extended')['tweet_cosine_similarity_to_neighbours']
			except:
				return False
				
		
			
	def getTweetIntegrityScore(self):
		if 'tweet_integrity_score' in self.getattr('extended'):
			return self.getattr('extended')['tweet_integrity_score']
		try:
			self.getattr('extended')['tweet_integrity_score'] = self.getLifetime()*self.getCosineSimilarity()
			return self.getattr('extended')['tweet_integrity_score']
		except:
			return False
			
	def getTweetInterestScore(self):
		if 'tweet_interest_score' in self.getattr('extended'):
			return self.getattr('extended')['tweet_interest_score']
		try:
			baseValue = 0
			if self.getattr('extended')['precision'] != 0:
				for k,x in self.getattr('extended')['precision'].iteritems():
					if x != 0: 
						if baseValue == 0: baseValue = 1
						baseValue *= x 
			else:
				baseValue = 1
			self.getattr('extended')['tweet_interest_score'] = baseValue *(self.getattr('retweet_count') if self.getattr('retweet_count') > 0 else 1)
			
			
			return self.getattr('extended')['tweet_interest_score']	
		except:
			return False
			
			
	def getAuthorIntegrityScore(self):
		if 'author_integrity_score' in self.getattr('extended'):
			return self.getattr('extended')['author_integrity_score']
		try:
			self.getattr('extended')['author_integrity_score'] = (self.getEstablishedScore() * self.getattr('user')['followers_count'])*(1+(1 if self.getattr('user')['verified'] == True else 0))
			return self.getattr('extended')['author_integrity_score']
		except:
			return False
		
	def getAuthorInterestScore(self):	
		if 'author_interest_score' in self.getattr('extended'):
			return self.getattr('extended')['author_interest_score']
		try:
			self.getattr('extended')['author_interest_score'] = (self.getReachScore() * (self.getattr('user')['listed_count'] if self.getattr('user')['listed_count'] > 0 else 1)) * (1+(1 if self.getattr('user')['following'] == True else 0))
			return self.getattr('extended')['author_interest_score']
		except:
			return False
			
			

class TwitterConnection(UserService):
	_sourceList = {
		'mentions_timeline' : 'statuses/mentions_timeline',
		'user_timeline' : 'statuses/user_timeline',
		'home_timeline' : 'statuses/home_timeline',
		'retweets_of_me' : 'statuses/retweets_of_me',
		'search_tweets' : 'search/tweets',
	}
	
	_comparablesList = {
		'mentions_timeline' : 'statuses/mentions_timeline',
		'user_timeline' : 'statuses/user_timeline',
		'home_timeline' : 'statuses/home_timeline',
		'retweets_of_me' : 'statuses/retweets_of_me',
		#'statuses/retweets/',
		'search_tweets' : 'search/tweets',
		'favorites' : 'favorites/list',
		'statuses_of_list' : 'lists/statuses',
		'trends' : 'trends/place.json',	
	}
	
	def __init__(self, userobj):	#API_KEY HERE TODO
		self.library = twitter.Twitter (
			api_version='1.1',
			auth=twitter.oauth.OAuth(
				userobj.oauthtoken,
				userobj.oauthsecret,
				'TwBgKt5194KFgwrbskRpA',
				'ECtwrTl1gWaLC9Ja0gAhyyPtpd1dXOVDJgbMz6U'
			)
		)
		
	def library_function(self, function, variables={}):
		try:
			a= getattr(self.library,function)(**variables)
			return a
		except:
			return False
			
			
	def parseSource(self, function, variables={}):
		try:
			function = function.strip()
			valid = False
			for k,v in self._sourceList.iteritems():
				if function == k:
					function = v
					valid = True
				elif function == v: 
					valid = True				
			if valid == False: raise Exception()
			
			response = self.library_function(function, variables)	
			if 'statuses' in response: response = response['statuses']
			return response
		except:
			return False
		
		
	def parseKeywords(self, function, variables={}):#for comparable sets
		try:
			function = function.strip()
			valid = False
			for k,v in self._comparablesList.iteritems():
				if function == k:
					function = v
					valid = True
				elif function == v: 
					valid = True				
			if valid == False: raise Exception()

			response = self.library_function(function, variables)
			
			k = KeywordExtraction()
			decode = function.split('/')
			if decode[0] in ['statuses','favorites','lists']:
				theString = ['.'+x['text'] for x in response]
				theString = ' '.join(theString)
				keywords = k.extractLiteral(theString)
			elif decode[0] == 'search':
				theString = ['.'+x['text'] for x in response['statuses']]
				theString = ' '.join(theString)
				keywords = k.extractLiteral(theString)
			elif decode[0] == 'trends': #needs testing
				keywords = [unicode(unicodedata.normalize('NFKD', x['name']).encode('ascii', 'ignore')) for x in response['trends']]
			else: raise Exception()	
			return keywords
		except:
			return []
		
	def extendedSourceInfo(self,source,comparableSets={},ranking=True):
		#print source
		source=list(source)
		tempTweetStore = []
		tweetList = TastyPieCollection()
		for k,tweet in enumerate(source):
			tweetList.data[k] = Tweet(tweet)
			tweetList.data[k].setattr('extended',{})
			tweetList.data[k].setattr('original',tweet)
			tweetList.data[k].getattr('text')
			tweetList.data[k].getLifetime()
			tweetList.data[k].getKeywords()
			
			tweetList.data[k].getAuthorAge()
			tweetList.data[k].getPostFrequency()
			tweetList.data[k].getEstablishedScore()
			tweetList.data[k].getReachScore()
			
			tweetList.data[k].getattr('extended')['precision'] = {}
			for o,v in comparableSets.iteritems():
				v = [x.lower() for x in v]
				tweetList.data[k].getattr('extended')['precision']['tweet_precision_'+o] = tweetList.data[k].getPrecision(tweetList.data[k].getKeywords(),v)
			
			tempTweetStore.append(tweetList.data[k].getattr('text'))
		[tweet.getCosineSimilarity(tempTweetStore) for tweet in tweetList.data.itervalues()] #Fixed where comparisions were being made on tweet.lower against original tweets. also float() for est and reach score

		if ranking == True:
			for tweet in tweetList.data.itervalues():
				tweet.getAuthorIntegrityScore()
				tweet.getAuthorInterestScore()
				tweet.getTweetIntegrityScore()
				tweet.getTweetInterestScore()
	
			#print tweetList.data[0].__dict__['extended']

			tweetList.data = nsga.nsgaSort(tweetList.data.values(),['author_interest_score','author_integrity_score'])
			tweetList.data = nsga.nsgaSort(tweetList.data.values(),['tweet_interest_score','tweet_integrity_score'])

		return tweetList

