from twitter import *
#import twitter
#from twitter.oauth_dance import parse_oauth_tokens as oauth_dance
from time import mktime, strptime, strftime, time, gmtime
from lib_generic import JsonDecoder,  TextContainer
from lib_keyword import KeywordExtraction
import json
import numpy
from operator import add
from cosine_test import process
import nsga


class Tweet(TextContainer):
	def getLifetime(self):
		if hasattr(self, 'lifetime'):
			return self.lifetime
		else:	
			try:
				setattr(self, 'lifetime', (int(time()) - int(mktime(strptime(self.created_at, '%a %b %d %H:%M:%S +0000 %Y')))))
				return self.lifetime
			except:
				return False
				
	def getAuthorAge(self):
		if hasattr(self.user, 'account_age'):
			return self.user.account_age		
		else:
			try:
				setattr(self.user, 'account_age', ((((int(time()) - int(mktime(strptime(self.user.created_at, '%a %b %d %H:%M:%S +0000 %Y'))))/60)/60)/24)	)
				return self.user.account_age				
			except:
				return False
				
	def getKeywords(self):
		if hasattr(self, 'keywords'):
			return self.keywords
		else:	
			try:
				k = KeywordExtraction()
				setattr(self, 'keywords', [kw.text.lower() for kw in (k.extractLiteral(self.text)).itervalues()])
				return self.keywords
			except:
				return False
				
	def getPrecision(self, setA, setB):	
		try:
			setA = set(setA)
			setB = set(setB)
			return len(setB.intersection(setA))/float(len(setA))
		except:
			return False
			
	def getEstablishedScore(self):
		if hasattr(self.user, 'establishment_score'):
			return self.user.establishment_score		
		else:
			try:
				setattr(self.user, 'establishment_score', (self.getAuthorAge()/self.user.friends_count))
				return self.user.establishment_score
			except ZeroDivisionError:
				setattr(self.user, 'establishment_score', 0)
				return self.user.establishment_score
			except:
				return False
				
	def getPostFrequency(self):
		if hasattr(self.user, 'post_frequency'):
			return self.user.post_frequency		
		else:
			try: 
				setattr(self.user, 'post_frequency', (self.user.statuses_count/self.getAuthorAge()))
				return self.user.post_frequency
			except ZeroDivisionError:
				setattr(self.user, 'post_frequency', 0)
				return self.user.post_frequency
			except:
				return False	
	
	def getReachScore(self):
		if hasattr(self.user, 'reach_score'):
			return self.user.reach_score		
		else:
			try:
				setattr(self.user, 'reach_score', (self.user.followers_count/self.getPostFrequency()))
				return self.user.reach_score
			except ZeroDivisionError:
				setattr(self.user, 'reach_score', 0)
				return self.user.reach_score
			except:
				return False
				
	def getCosineSimilarity(self,againstList=None):
		if hasattr(self, 'cosine_similarity_to_neighbours'):
			return self.cosine_similarity_to_neighbours
		else:
			if not isinstance(againstList,list) or len(againstList) == 0:
				return False
			try:
				cosineAverages = [process(againstText,self.getStaticAttribute('text').lower()) if (self.getStaticAttribute('text') != againstText) else False for againstText in againstList]
				setattr(self, 'cosine_similarity_to_neighbours', round(numpy.average(cosineAverages),4))
				return self.cosine_similarity_to_neighbours
			except:
				return False
				
		
			
	def getTweetIntegrityScore(self):
		if hasattr(self, 'tweet_integrity_score'):
			return self.tweet_integrity_score
		try:
			setattr(self, 'tweet_integrity_score', self.getLifetime()*self.getCosineSimilarity())
			return self.tweet_integrity_score
		except:
			return False
			
	def getTweetInterestScore(self):
		if hasattr(self, 'tweet_interest_score'):
			return self.tweet_interest_score
		try:
			baseValue = 0
			for k,x in self.getStaticAttribute('precision').__dict__.iteritems():
				if x != 0: 
					if baseValue == 0: baseValue = 1
					baseValue *= x 
				setattr(self, 'tweet_interest_score', baseValue *(self.getStaticAttribute('retweet_count') if self.getStaticAttribute('retweet_count') > 0 else 1))
			return self.tweet_interest_score		
		except:
			return False
			
			
	def getAuthorIntegrityScore(self):
		if hasattr(self, 'author_integrity_score'):
			return self.author_integrity_score
		try:
			setattr(self, 'author_integrity_score', (self.getEstablishedScore() * self.getStaticAttribute('user').getStaticAttribute('followers_count'))*(1+(1 if self.getStaticAttribute('user').getStaticAttribute('verified') == True else 0)))
			return self.author_integrity_score
		except:
			return False
		
	def getAuthorInterestScore(self):	
		if hasattr(self, 'author_interest_score'):
			return self.author_interest_score
		try:
			setattr(self, 'author_interest_score', (self.getReachScore() * (self.getStaticAttribute('user').getStaticAttribute('listed_count') if self.getStaticAttribute('user').getStaticAttribute('listed_count') > 0 else 1)) * (1+(1 if self.getStaticAttribute('user').getStaticAttribute('following') == True else 0)))
			return self.author_interest_score
		except:
			return False
			
			

class TweetCollection(object):
	def __init__(self,tweetList):
		if (isinstance(tweetList,list)):
			self.tweets=[]
			tempTweetStore=[]
			
			for tweet in tweetList:
				self.tweets.append(Tweet(tweet))
				self.tweets[-1].getLifetime()
				self.tweets[-1].getKeywords()
				
				self.tweets[-1].getAuthorAge()
				self.tweets[-1].getPostFrequency()
				self.tweets[-1].getEstablishedScore()
				self.tweets[-1].getReachScore()
				tempTweetStore.append(self.tweets[-1].getStaticAttribute('text'))		
			[tweet.getCosineSimilarity(tempTweetStore) for tweet in self.tweets]
		
	def getPrecisionAgainst(self,comparableSets={}):

		"""comparableSets = json.dumps({"search": ["a","b","c","d"],"new": ["1","2","3","4","tomatoes"]})
		
		oj = json.dumps({"statuses":[{"a":"b"},{"text":"s"}]})
		oj = json2.decode(oj)
		tweetList = oj['statuses']"""
		for tweet in self.tweets:
			if not hasattr(tweet,'precision'):
				setattr(tweet,'precision',TextContainer({}))
			for k,v in comparableSets.iteritems():
				if not hasattr(tweet.precision,k+'_precision'):
					setattr(tweet.precision, k+'_precision', tweet.getPrecision(tweet.getKeywords(),v))
					
		return self.tweets
		
	def getRankingScores(self):
		for tweet in self.tweets:
			tweet.getTweetIntegrityScore()	
			tweet.getTweetInterestScore()
			tweet.getAuthorIntegrityScore()
			tweet.getAuthorInterestScore()
		return self.tweets

		

"""
app_name = "Chimaera Project"
consumer_key = "TwBgKt5194KFgwrbskRpA"
consumer_secret = "ECtwrTl1gWaLC9Ja0gAhyyPtpd1dXOVDJgbMz6U"


twitterConnect = twitter.Twitter(
	api_version=None,
	auth=twitter.oauth.OAuth(
		'',
		'',
		consumer_key,9.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519]]
[29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519] belongs in Front: 1
[29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519] dominates: [[29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519]]
[29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519] belongs in Front: 1
[29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519] dominates: [[29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.07150000000001, 149.07150000000001, 151.9276, 151.9276, 174.048, 174.048, 163.9858, 163.9858, 180.9332, 180.9332, 185.248, 185.248, 190.54610000000002, 190.54610000000002, 194.91000000000003, 194.91000000000003, 216.70170000000002, 216.70170000000002, 192.6336, 192.6336, 213.8895, 213.8895, 213.17610000000002, 213.17610000000002, 229.2331, 229.2331, 235.91539999999998, 235.91539999999998, 235.5222, 235.5222, 245.71359999999999, 245.71359999999999, 277.7132, 277.7132, 286.546, 286.546, 302.99780000000004, 302.99780000000004, 340.2564, 340.2564, 359.98800000000006, 359.98800000000006, 382.95469999999995, 382.95469999999995, 386.59979999999996, 386.59979999999996, 402.896, 402.896, 421.7096, 421.7096, 386.835, 386.835, 336.846, 336.846, 433.7194, 433.7194, 430.13789999999995, 430.13789999999995, 463.5189, 463.5189, 466.83959999999996, 466.83959999999996, 449.0376, 449.0376, 450.5655, 450.5655, 431.8519, 431.8519], [29.8364, 29.8364, 31.8964, 31.8964, 64.63, 64.63, 61.26840000000001, 61.26840000000001, 79.35560000000001, 79.35560000000001, 85.692, 85.692, 85.617, 85.617, 88.8888, 88.8888, 88.01759999999999, 88.01759999999999, 85.3335, 85.3335, 110.6474, 110.6474, 107.56760000000001, 107.56760000000001, 73.908, 73.908, 116.99839999999999, 116.99839999999999, 129.0096, 129.0096, 143.0564, 143.0564, 149.0
		consumer_secret
	),
	format=''
)


oauth_token, oauth_token_secret = oauth_dance(twitterConnect.oauth.request_token())

oauth_url = ('http://api.twitter.com/oauth/authorize?oauth_token=' + oauth_token)


oauth_verifier = '1040088'

twitterConnect = twitter.Twitter(
	api_version=None,
	auth=twitter.oauth.OAuth(
		oauth_token,
		oauth_token_secret,
		consumer_key,
		consumer_secret
	),
	format=''
)

#oauth_token, oauth_token_secret = oauth_dance(twitterConnect.oauth.access_token(oauth_verifier=oauth_verifier))

print oauth_url
webbrowser.open(oauth_url)
"""

"""
connection = Twitter (
	api_version='1.1',
	auth=OAuth(
		'587307932-aYEfHGvWmg3O7VMmUOz0zAg1ki4RP6mLVAkJ6PfQ',
		'IMFBoQ0huqWWqdYTDSjoYpxZ5dzHoWVlFbh9SWvHFnI',
		'TwBgKt5194KFgwrbskRpA',
		'ECtwrTl1gWaLC9Ja0gAhyyPtpd1dXOVDJgbMz6U'
	)
)

searchTerms = ['Tomatoes']
a = connection.search.tweets(q=(','.join(searchTerms)),lang='en',count=10)
TC = TweetCollection(a['statuses'])
		
jsonDecoder = JsonDecoder()
tweets = TC.getPrecisionAgainst(jsonDecoder.decode(json.dumps({"search": ["a","b","c","Tomatoes"],"new": ["1","2","3","4","tomatoes"]})))
TC.getRankingScores()
nsga.nsgaSort(tweets,['author_interest_scorde','tweet_integrity_score'])
"""
